/* -------------------------------------------------------------------------
 * Copyright (c) 2009 Steve Jolly. All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the COPYING file for a copy of the GNU General Public Licence.
 *
 * -------------------------------------------------------------------------
 */
package net.elvum.hpgen;

import java.io.UnsupportedEncodingException;

/**
 * A utitlity class containing static methods for converting things.
 * @author Steve Jolly
 */
public class Conversions {

    private static byte[] base64Map;
    private static byte equalsByte;

    static {
        String base64chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        try {
            base64Map = base64chars.getBytes("US-ASCII");
            equalsByte = "=".getBytes("US-ASCII")[0];
        } catch (UnsupportedEncodingException ex) {
            ex.printStackTrace();
        }
    }

    private Conversions() {
        //we don't want no steenking initialisations
    }

    public static String toBase64(byte[] original) {
        byte[] result = new byte[((original.length + 2) / 3) * 4];
        int outputLength = (original.length * 4 + 2) / 3; //unpadded
        int ip = 0, rp = 0, i0, i1, i2, r0, r1, r2, r3;
        while (ip < original.length) {
            i0 = original[ip++] & 0xff;
            i1 = ip < original.length ? original[ip++] & 0xff : 0;
            i2 = ip < original.length ? original[ip++] & 0xff : 0;
            r0 = i0 >>> 2;
            r1 = ((i0 & 3) << 4) | (i1 >>> 4);
            r2 = ((i1 & 0xf) << 2) | (i2 >>> 6);
            r3 = i2 & 0x3F;
            result[rp++] = base64Map[r0];
            result[rp++] = base64Map[r1];
            result[rp] = rp < outputLength ? base64Map[r2] : equalsByte;
            rp++;
            result[rp] = rp < outputLength ? base64Map[r3] : equalsByte;
            rp++;
        }
        try {
            return new String(result, "US-ASCII");
        } catch (UnsupportedEncodingException ex) {
            ex.printStackTrace();
            return null;
        }
    }
}
